@@ -1,7 +1,6 @@
 /* Modula 2 language support routines for GDB, the GNU debugger.
 
-   Copyright (C) 1992, 1993, 1994, 1995, 1996, 1998, 2000, 2002, 2003, 2004,
-   2005, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+   Copyright (C) 1992-2013 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -42,7 +41,7 @@ static void
 m2_emit_char (int c, struct type *type, struct ui_file *stream, int quoter)
 {
 
-  c &= 0xFF;			/* Avoid sign bit follies */
+  c &= 0xFF;			/* Avoid sign bit follies.  */
 
   if (PRINT_LITERAL_FORM (c))
     {
@@ -111,7 +110,6 @@ m2_printstr (struct ui_file *stream, struct type *type, const gdb_byte *string,
   unsigned int things_printed = 0;
   int in_quotes = 0;
   int need_comma = 0;
-  int width = TYPE_LENGTH (type);
 
   if (length == 0)
     {
@@ -147,10 +145,7 @@ m2_printstr (struct ui_file *stream, struct type *type, const gdb_byte *string,
 	{
 	  if (in_quotes)
 	    {
-	      if (options->inspect_it)
-		fputs_filtered ("\\\", ", stream);
-	      else
-		fputs_filtered ("\", ", stream);
+	      fputs_filtered ("\", ", stream);
 	      in_quotes = 0;
 	    }
 	  m2_printchar (string[i], type, stream);
@@ -163,10 +158,7 @@ m2_printstr (struct ui_file *stream, struct type *type, const gdb_byte *string,
 	{
 	  if (!in_quotes)
 	    {
-	      if (options->inspect_it)
-		fputs_filtered ("\\\"", stream);
-	      else
-		fputs_filtered ("\"", stream);
+	      fputs_filtered ("\"", stream);
 	      in_quotes = 1;
 	    }
 	  LA_EMIT_CHAR (string[i], type, stream, '"');
@@ -176,12 +168,7 @@ m2_printstr (struct ui_file *stream, struct type *type, const gdb_byte *string,
 
   /* Terminate the quotes if necessary.  */
   if (in_quotes)
-    {
-      if (options->inspect_it)
-	fputs_filtered ("\\\"", stream);
-      else
-	fputs_filtered ("\"", stream);
-    }
+    fputs_filtered ("\"", stream);
 
   if (force_ellipses || i < length)
     fputs_filtered ("...", stream);
@@ -195,6 +182,7 @@ evaluate_subexp_modula2 (struct type *expect_type, struct expression *exp,
   struct value *arg1;
   struct value *arg2;
   struct type *type;
+
   switch (op)
     {
     case UNOP_HIGH:
@@ -211,6 +199,7 @@ evaluate_subexp_modula2 (struct type *expect_type, struct expression *exp,
 	  if (m2_is_unbounded_array (type))
 	    {
 	      struct value *temp = arg1;
+
 	      type = TYPE_FIELD_TYPE (type, 1);
 	      /* i18n: Do not translate the "_m2_high" part!  */
 	      arg1 = value_struct_elt (&temp, NULL, "_m2_high", NULL,
@@ -240,10 +229,12 @@ evaluate_subexp_modula2 (struct type *expect_type, struct expression *exp,
 	{
 	  struct value *temp = arg1;
 	  type = TYPE_FIELD_TYPE (type, 0);
-	  if (type == NULL || (TYPE_CODE (type) != TYPE_CODE_PTR)) {
-	    warning (_("internal error: unbounded array structure is unknown"));
-	    return evaluate_subexp_standard (expect_type, exp, pos, noside);
-	  }
+	  if (type == NULL || (TYPE_CODE (type) != TYPE_CODE_PTR))
+	    {
+	      warning (_("internal error: unbounded "
+			 "array structure is unknown"));
+	      return evaluate_subexp_standard (expect_type, exp, pos, noside);
+	    }
 	  /* i18n: Do not translate the "_m2_contents" part!  */
 	  arg1 = value_struct_elt (&temp, NULL, "_m2_contents", NULL,
 				   _("unbounded structure "
@@ -252,7 +243,7 @@ evaluate_subexp_modula2 (struct type *expect_type, struct expression *exp,
 	  if (value_type (arg1) != type)
 	    arg1 = value_cast (type, arg1);
 
-	  type = check_typedef (value_type (arg1));
+	  check_typedef (value_type (arg1));
 	  return value_ind (value_ptradd (arg1, value_as_long (arg2)));
 	}
       else
@@ -356,6 +347,7 @@ const struct exp_descriptor exp_descriptor_modula2 =
 {
   print_subexp_standard,
   operator_length_standard,
+  operator_check_standard,
   op_name_standard,
   dump_subexp_body_standard,
   evaluate_subexp_modula2
@@ -366,7 +358,6 @@ const struct language_defn m2_language_defn =
   "modula-2",
   language_m2,
   range_check_on,
-  type_check_on,
   case_sensitive_on,
   array_row_major,
   macro_expansion_no,
@@ -381,12 +372,14 @@ const struct language_defn m2_language_defn =
   m2_print_typedef,		/* Print a typedef using appropriate syntax */
   m2_val_print,			/* Print a value using appropriate syntax */
   c_value_print,		/* Print a top-level value */
+  default_read_var_value,	/* la_read_var_value */
   NULL,				/* Language specific skip_trampoline */
   NULL,		                /* name_of_this */
   basic_lookup_symbol_nonlocal,	/* lookup_symbol_nonlocal */
   basic_lookup_transparent_type,/* lookup_transparent_type */
   NULL,				/* Language specific symbol demangler */
-  NULL,				/* Language specific class_name_from_physname */
+  NULL,				/* Language specific
+				   class_name_from_physname */
   m2_op_print_tab,		/* expression operators for printing */
   0,				/* arrays are first-class (not c-style) */
   0,				/* String lower bound */
@@ -396,6 +389,8 @@ const struct language_defn m2_language_defn =
   default_print_array_index,
   default_pass_by_reference,
   default_get_string,
+  NULL,				/* la_get_symbol_name_cmp */
+  iterate_over_symbols,
   LANG_MAGIC
 };
 
